using DataFrames
using XLSX
using JuMP
using Gurobi
using Gadfly

slats_lengths = XLSX.readdata("MM1_Dataset2.xlsx", "Slats", "A6:B155")
panels_dim = XLSX.readdata("MM1_Dataset2.xlsx", "Panels", "A4:D8")

#data
w = XLSX.readdata("MM1_Dataset2.xlsx", "Slats", "B3")
Np = panels_dim[:,4]


#Sets
S = slats_lengths[:,1]
P = panels_dim[:,1]
st = copy(Np)


#Parameters
l = slats_lengths[:,2]
L = panels_dim[:,2]
W = panels_dim[:,3]


#Model definition
model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>120))


#Variables definition
#X_p,st,s : binary variable, equal to 1 if slat s is used in strip st of panel p
@variable(model, X[p = 1:length(P), st = 1:Np[p], s = 1:length(S)], Bin)


#Objective function
@objective(model, Min, sum(sum(X[p,st,s]*l[s] for s=1:length(S)) - L[p] for p=1:length(P) for st=1:Np[p])
                        + sum(L[p]*((Np[p]*w)-W[p]) for p=1:length(P)))

#Constraints
#1 A slat can only be used once
for s=1:length(S)
        @constraint(model, sum(X[p,st,s] for p = 1:length(P) for st = 1:Np[p]) <= 1)
end

#2 The length of each strip must be longer or equal to the length of the panel p
for p = 1:length(P), st = 1:Np[p]
        @constraint(model, sum(X[p,st,s]*l[s] for s=1:length(S)) >= L[p])
end


#Solve the model
status = optimize!(model)

#Value of the objective function
total_waste = objective_value(model)

#Retrieving X
Xsol = value.(X)

#Create the DataFrame in which you would like to store the solution
X_VALUES = DataFrame(Panel = [], Strip = [], Slat = [], Length = [])

#Adding the solutions in the DataFrames
#X
for p = 1:length(P)
    for st = 1:Np[p]
        for s = 1:length(S)
            val = Xsol[p,st,s]
            if val == 1
                length = l[s]
                push!(X_VALUES, vcat(p, st, s, length))
            end
        end
    end
end
X_VALUES


#Export the DataFrames into Excel files
XLSX.writetable("MM1_Sol_Dataset2.xlsx", collect(DataFrames.eachcol(X_VALUES)), DataFrames.names(X_VALUES))
ng XLSX
using JuMP
using Gurobi
using Gadfly

slats_lengths = XLSX.readdata("MM1_Dataset2.xlsx", "Slats", "A6:B155")
panels_dim = XLSX.readdata("MM1_Dataset2.xlsx", "Panels", "A4:D8")

#data
w = XLSX.readdata("MM1_Dataset2.xlsx", "Slats", "B3")
Np = panels_dim[:,4]


#Sets
S = slats_lengths[:,1]
P = panels_dim[:,1]
st = copy(Np)


#Parameters
l = slats_lengths[:,2]
L = panels_dim[:,2]
W = panels_dim[:,3]


#Model definition
model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>120))


#Variables definition
#X_p,st,s : binary variable, equal to 1 if slat s is used in strip st of panel p
@variable(model, X[p = 1:length(P), st = 1:Np[p], s = 1:length(S)], Bin)


#Objective function
@objective(model, Min, sum(sum(X[p,st,s]*l[s] for s=1:length(S)) - L[p] for p=1:length(P) for st=1:Np[p])
                        + sum(L[p]*((Np[p]*w)-W[p]) for p=1:length(P)))

#Constraints
#1 A slat can only be used once
for s=1:length(S)
        @constraint(model, sum(X[p,st,s] for p = 1:length(P) for st = 1:Np[p]) <= 1)
end

#2 The length of each strip must be longer or equal to the length of the panel p
for p = 1:length(P), st = 1:Np[p]
        @constraint(model, sum(X[p,st,s]*l[s] for s=1:length(S)) >= L[p])
end


#Solve the model
status = optimize!(model)

#Value of the objective function
total_waste = objective_value(model)

#Retrieving X
Xsol = value.(X)

#Create the DataFrame in which you would like to store the solution
X_VALUES = DataFrame(Panel = [], Strip = [], Slat = [], Length = [])

#Adding the solutions in the DataFrames
#X
for p = 1:length(P)
    for st = 1:Np[p]
        for s = 1:length(S)
            val = Xsol[p,st,s]
            if val == 1
                length = l[s]
                push!(X_VALUES, vcat(p, st, s, length))
            end
        end
    end
end
X_VALUES


#Export the DataFrames into Excel files
XLSX.writetable("MM1_Sol_Dataset2.xlsx", collect(DataFrames.eachcol(X_VALUES)), DataFram