using DataFrames
using XLSX
using JuMP
using Gurobi
using Gadfly

File = "MM2_Dataset3" #Choose the data file

slats_lengths = XLSX.readdata("$File.xlsx", "Slats", "A6:B155")
panels_dim = XLSX.readdata("$File.xlsx", "Panels", "B4:D4")

#data
w = XLSX.readdata("$File.xlsx", "Slats", "B3")
Np = panels_dim[3]
L = panels_dim[1]
W = panels_dim[2]

#Sets
S = slats_lengths[:,1]
P = Int64(floor(length(S)/Np))
ST = copy(Np)


#Parameters
l = slats_lengths[:,2]


#Model definition
model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>3600))


#Variables definition
#X_p,st,s : binary variable, equal to 1 if slat s is used in strip st of panel p
@variable(model, X[p = 1:P, st = 1:ST, s = 1:length(S)], Bin)
@variable(model, Y[p = 1:P], Bin)


#Objective function
@objective(model, Max, sum(Y[p] for p = 1:P))

#Constraints
#1 All slats can only be used once
for s=1:length(S)
        @constraint(model, sum(X[p,st,s] for p = 1:P for st = 1:ST) <= 1)
end

#2 The length of each strip must be longer or equal to the length of a panel
for p = 1:P, st = 1:ST
        @constraint(model, sum(X[p,st,s]*l[s] for s=1:length(S)) >= L*Y[p])
end


#Solve the model
status = optimize!(model)

#Value of the objective function
nb_panel = objective_value(model)

Comp_time = round(solve_time(model))

best_bound = objective_bound(model)


#Retrieving X & Y
Xsol = value.(X)
Ysol = value.(Y)

#Panels created
Ypanel = Vector{Int64}()
for i = 1:length(Ysol)
    r = Ysol[i]
        if r == 1
        push!(Ypanel, i)
    end
end
Ypanel


#Create the DataFrame in which you would like to store the solution
X_VALUES = DataFrame(Panel = [], Strip = [], Slat = [], Length = [])

#Adding the solutions in the DataFrames
#X
for p = 1:P
    for st = 1:ST
        for s = 1:length(S)
            if Ysol[p] == 1
                val = Xsol[p,st,s]
                if val == 1
                    length = l[s]
                    push!(X_VALUES, vcat(p, st, s, length))
                end
            end
        end
    end
end
X_VALUES

#Waste
#"Constant" waste
Constant_waste = ((w*Np)-W)*L*nb_panel
#"Variable" waste
Variable_waste = (sum(X_VALUES[:,4]) - (nb_panel*Np*L))*w
#Total waste
Total_waste = Constant_waste + Variable_waste


#Export the DataFrames into Excel files
XLSX.writetable("MM2_Sol_Dataset3.xlsx", collect(DataFrames.eachcol(X_VALUES)), DataFrames.names(X_VALUES))



#Parameters
l = slats_lengths[:,2]


#Model definition
model = Model(optimizer_with_attributes(Gurobi.Optimizer,"TimeLimit"=>3600))


#Variables definition
#X_p,st,s : binary variable, equal to 1 if slat s is used in strip st of panel p
@variable(model, X[p = 1:P, st = 1:ST, s = 1:length(S)], Bin)
@variable(model, Y[p = 1:P], Bin)


#Objective function
@objective(model, Max, sum(Y[p] for p = 1:P))

#Constraints
#1 All slats can only be used once
for s=1:length(S)
        @constraint(model, sum(X[p,st,s] for p = 1:P for st = 1:ST) <= 1)
end

#2 The length of each strip must be longer or equal to the length of a panel
for p = 1:P, st = 1:ST
        @constraint(model, sum(X[p,st,s]*l[s] for s=1:length(S)) >= L*Y[p])
end


#Solve the model
status = optimize!(model)

#Value of the objective function
nb_panel = objective_value(model)

Comp_time = round(solve_time(model))

best_bound = objective_bound(model)


#Retrieving X & Y
Xsol = value.(X)
Ysol = value.(Y)

#Panels created
Ypanel = Vector{Int64}()
for i = 1:length(Ysol)
    r = Ysol[i]
        if r == 1
        push!(Ypanel, i)
    end
end
Ypanel


#Create the DataFrame in which you would like to store the solution
X_VALUES = DataFrame(Panel = [], Strip = [], Slat = [], Length = [])

#Adding the solutions in the DataFrames
#X
for p = 1:P
    for st = 1:ST
        for s = 1:length(S)
            if Ysol[p] == 1
                val = Xsol[p,st,s]
                if val == 1
                    length = l[s]
                    push!(X_VALUES, vcat(p, st, s, length))